@@ -52,29 +52,6 @@ download <- function(url,
 }
 
 downloadImpl <- function(url, method, ...) {
-
-  # If this is a path to a GitHub URL, attempt to download with authentication,
-  # so that private GitHub repositories can be handled.
-  if (isGitHubURL(url) && canUseGitHubDownloader()) {
-    result <- try(githubDownload(url, ...), silent = TRUE)
-    if (!inherits(result, "try-error"))
-      return(result)
-  }
-
-  # If this is a path to a Bitbucket URL, attempt to download.
-  if (isBitbucketURL(url) && canUseBitbucketDownloader()) {
-    result <- try(bitbucketDownload(url, ...), silent = TRUE)
-    if (!inherits(result, "try-error"))
-      return(result)
-  }
-
-  # If this is a path to a Gitlab URL, attempt to download.
-  if (isGitlabURL(url) && canUseGitlabDownloader()) {
-    result <- try(gitlabDownload(url, ...), silent = TRUE)
-    if (!inherits(result, "try-error"))
-      return(result)
-  }
-
   # When on Windows using an 'internal' method, we need to call
   # 'setInternet2' to set some appropriate state.
   if (is.windows() && method == "internal") {
@@ -166,17 +143,10 @@ downloadFile <- function(url,
 
 }
 
-# TODO: we cannot use libcurl until R no longer sends HEAD requests when
-# attempting to download files
-canUseLibCurlDownloadMethod <- function() {
-  FALSE
-}
-
-
 # Attempt download.packages multiple times.
 #
 # Assumes we are downloading a single package.
-downloadPackagesWithRetries <- function(name, destdir, repos, type, maxTries = 5L) {
+downloadPackagesWithRetries <- function(name, destdir, repos, type, maxTries = 2L) {
   maxTries <- as.integer(maxTries)
   stopifnot(maxTries > 0L)
   stopifnot(length(name) > 0L)
@@ -195,10 +165,15 @@ downloadPackagesWithRetries <- function(name, destdir, repos, type, maxTries = 5
   fileLoc
 }
 
-
-# Download from a URL with a certain number of retries -- returns TRUE
-# if the download succeeded, and FALSE otherwise
-downloadWithRetries <- function(url, ..., maxTries = 5L) {
+# Download from a URL with a certain number of retries.
+# - Generic download function, used if we have no provider-specific
+#   authentication or cannot call those functions for other reasons.
+# - Wraps the inner `download` function in a retry pattern. Will raise errors
+#   that come from that function, and will also raise an error if the `success`
+#   marker is not `TRUE` after `maxTries` attempts.
+# - Returns `TRUE` if its success marker is `TRUE`.
+# - Passes `destfile` down to `download` via `...`.
+downloadWithRetries <- function(url, ..., maxTries = 2L) {
   maxTries <- as.integer(maxTries)
   stopifnot(maxTries > 0L)
 
@@ -221,7 +196,47 @@ downloadWithRetries <- function(url, ..., maxTries = 5L) {
     )
     if (success) break
   }
-  success
+  if (!success) {
+    stop("Download failed.")
+  }
+  return(TRUE)
+}
+
+canUseRenvDownload <- function() {
+  getOption("packrat.authenticated.downloads.use.renv", FALSE) &&
+    identical(secureDownloadMethod(), "curl")
+}
+
+# - Called by git provider download (e.g. `githubDownload`) when the option
+#   `"packrat.authenticated.downloads.use.renv"` is set to `TRUE` and
+#   `secureDownloadMethod` returns `"curl"`. Wraps `renv$download`.
+# - `type` is a lowercase string that should match `"github"`, `"gitlab"`, or
+#   `"bitbucket"`. `renv` uses this to determine what type of authentication to
+#   use. This is hard-coded in the provider download functions.
+# - Raises errors generated by `renv$download`. In addition, if `renv$download`
+#   finishes without generating an error, but no file exists at `destfile`,
+#   raises an error. Otherwise, returns `TRUE` (for consistency with other
+#   download functions, but not checked by provider downloaders).
+# - `renv$download` writes to `destfile`. The lifecycle of this function is
+#   managed by `getSourceForPkgRecord`. `renv` writes to another temporary file
+#   internally in some instances, but we don't need to worry about that.
+renvDownload <- function(url, destfile, method = inferAppropriateDownloadMethod(url), type = NULL, ...) {
+  if (identical(type, "bitbucket")) {
+    # We temporarily set our user agent to "curl" so that Bitbucket will treat us
+    # like a command line and not a browser. Otherwise, if we make unauthorized
+    # requests to Bitbucket .tar.gz URLs, we get redirects instead of a 401.
+    renv_useragent_option <- options("renv.http.useragent" = "curl")
+    on.exit(options(renv_useragent_option), add = TRUE)
+  }
+
+  result <- with_envvar(
+    c(RENV_DOWNLOAD_METHOD = method),
+    renv$download(url = url, destfile = destfile, type = type)
+  )
+  if (!file.exists(destfile)) {
+    stop("No data received.", call. = FALSE)
+  }
+  return(TRUE)
 }
 
 inferAppropriateDownloadMethod <- function(url) {
@@ -267,24 +282,19 @@ secureDownloadMethod <- function() {
     # that doesn't rely on the value of setInternet2). If it's R <= 3.1
     # then we can use "internal" for https so long as internet2 is enabled
     # (we don't use libcurl on Windows because it doesn't check certs).
-    if (isR32)
-        return("wininet")
-
-    # Otherwise, make a call to 'setInternet2' and use the 'internal' method
-    # if that call succeeds.
-    seti2 <- `::`(utils, 'setInternet2')
-    if (suppressWarnings(seti2(NA)))
-      return("internal")
+    if (isR32) {
+      return("wininet")
+    } else {
+      # Otherwise, make a call to 'setInternet2' and use the 'internal' method
+      # if that call succeeds.
+      seti2 <- `::`(utils, 'setInternet2')
+      if (suppressWarnings(seti2(NA))) {
+        return("internal")
+      }
+    }
   }
 
-  # For Darwin and Linux we use libcurl if we can and then fall back
-  # to curl or wget as appropriate. We prefer libcurl because it honors
-  # the same proxy configuration that "internal" does so it less likely
-  # to break downloads for users behind proxy servers.
-  if (canUseLibCurlDownloadMethod())
-    return("libcurl")
-
-  # Otherwise, fall back to 'wget' or 'curl' (preferring 'wget')
+  # Otherwise, fall back to 'wget' or 'curl' (preferring 'curl')
   candidates <- c("curl", "wget")
   for (candidate in candidates)
     if (isProgramOnPath(candidate))
@@ -292,3 +302,65 @@ secureDownloadMethod <- function() {
 
   stop("Failed to discover a secure download method.")
 }
+
+# Returns a function to be used as an error handler for githubDownload,
+# gitlabDownload, and bitbucketDownload.
+authDownloadAdvice <- function(type, authenticated, downloader) {
+  f <- function(e = NULL) {
+    advice <- paste0(
+      "If you are trying to restore a package from a private Git repo, ",
+      "you must have credentials available in your environment, ",
+      "and Packrat must be configured to use an auth-capable download method."
+    )
+
+    # Info on available auth tokens, dependant on provider type.
+    token_msg <- NULL
+    if (identical(type, "github")) {
+      if (identical(authenticated, TRUE)) {
+        token_msg <- "GITHUB_PAT found; check that it is correct."
+      } else {
+        token_msg <- "GITHUB_PAT environment variable not found."
+      }
+    }
+    if (identical(type, "gitlab")) {
+      if (identical(authenticated, TRUE)) {
+        token_msg <- "GITLAB_PAT found; check that it is correct."
+      } else {
+        token_msg <- "GITLAB_PAT environment variable not found."
+      }
+    }
+    if (identical(type, "bitbucket")) {
+      if (identical(authenticated, TRUE)) {
+        token_msg <- "BITBUCKET_USERNAME and BITBUCKET_PASSWORD found; check that they are correct."
+      } else {
+        token_msg <- "BITBUCKET_USERNAME and BITBUCKET_PASSWORD environment variables not found."
+      }
+    }
+    advice <- c(advice, token_msg)
+
+    # Info on configuration
+    if (identical(downloader, "renv")) {
+      advice <- c(advice, "Packrat is configured to use internal renv for authenticated downloads.")
+    } else if (identical(downloader, "httr")) {
+      advice <- c(advice, "Packrat will use the httr package for authenticated downloads.")
+    } else {
+      advice <- c(advice, paste0(
+        "Packrat is not configured to use an auth-capable download ",
+        "method. Try setting the option ",
+        "packrat.authenticated.downloads.use.renv to TRUE, or installing the ",
+        "httr package."
+      ))
+    }
+    advice <- paste(advice, collapse = " ")
+
+    # This allows us to see the advice without appending to an error message.
+    if (is.null(e)) {
+      return(advice)
+    } else {
+      e$message <- paste(e$message, advice, sep = "\n")
+      stop(e)
+    }
+  }
+
+  return(f)
+}
