@@ -186,44 +186,31 @@ getSourceForPkgRecord <- function(pkgRecord,
              "(", pkgRecord$version, ")")
         }
 
-      # If the file wasn't saved to the destination directory (which can happen
-      # if the repo is local--see documentation in download.packages), copy it
-      # there now
-      if (!identical(fileLoc[1, 2], file.path(pkgSrcDir, pkgSrcFile))) {
-        file.copy(fileLoc[1, 2], pkgSrcDir)
+      downloadedFile <- fileLoc[1, 2]
+      if (identical(dirname(downloadedFile), pkgSrcDir)) {
+        # The target file is in the destination directory.
+        if (!identical(basename(downloadedFile), pkgSrcFile)) {
+          # When the file has a different name, rename it. This can happen if the repository download redirects to a URL with a different path.
+          file.rename(downloadedFile, file.path(pkgSrcDir, pkgSrcFile))
+        }
+      } else {
+        # When target file is in some different directory, copy it. This can happen if the repo is local (see download.packages documentation).
+        file.copy(downloadedFile, file.path(pkgSrcDir, pkgSrcFile))
       }
       type <- paste(type, "current")
     } else {
       # The version requested is not the version on CRAN; it may be an
       # older version. Look for the older version in the CRAN archive for
       # each named repository.
-      foundVersion <- FALSE
-      for (repo in repos) {
-        tryCatch({
-          archiveUrl <- file.path(repo, "src/contrib/Archive",
-                                  pkgRecord$name,
-                                  pkgSrcFile)
-          downloadWithRetries(archiveUrl,
-                              destfile = file.path(pkgSrcDir, pkgSrcFile),
-                              mode = "wb", quiet = TRUE)
-          foundVersion <- TRUE
-          type <- paste(type, "archived")
-          break
-        }, error = function(e) {
-          # Ignore error and try the next repository
-        })
-      }
+      destFile <- file.path(pkgSrcDir, pkgSrcFile)
+      foundVersion <- downloadArchiveWithRetries(repos, pkgRecord, destFile)
       if (!foundVersion) {
         message("FAILED")
-        stopMsg <- sprintf("Failed to retrieve package sources for %s %s from CRAN (internet connectivity issue?)",
-                           pkgRecord$name,
-                           pkgRecord$version)
-
-        if (!is.na(currentVersion))
-          stopMsg <- paste(stopMsg, sprintf("[%s is current]", currentVersion))
-
-        stop(stopMsg)
+        stop(sprintf("Failed to retrieve package sources for %s %s from a package repository archive (internet connectivity issue?)",
+                     pkgRecord$name,
+                     pkgRecord$version))
       }
+      type <- paste(type, "archived")
     }
   } else if (identical(pkgRecord$source, "github")) {
     archiveUrl <- githubArchiveUrl(pkgRecord)
@@ -342,6 +329,60 @@ getSourceForPkgRecord <- function(pkgRecord,
   }
 }
 
+useAlternativeArchiveLocations <- function() {
+  getOption("packrat.alternative.archive.layouts", TRUE)
+}
+
+# Returns TRUE when the package is downloaded, FALSE otherwise.
+downloadArchiveWithRetries <- function(repos, pkgRecord, destFile) {
+  pkgSrcFile <- pkgSrcFilename(pkgRecord)
+
+  # Many repositories follow the CRAN structure, but not all. Try the CRAN
+  # structure across all configured repositories before attempting a
+  # different path.
+  #
+  # The renv implementation probes the repository to determine which
+  # formatter is appropriate. It previously attempted each possibility.
+  # https://github.com/rstudio/renv/blob/731463ec6a52588c148741d2e60c9107c339d941/R/retrieve.R#L951-L1021
+  formatters <- c(
+    # default CRAN format.
+    function(repo) {
+      file.path(repo, "src/contrib/Archive", pkgRecord$name, pkgSrcFile)
+    }
+  )
+
+  if (useAlternativeArchiveLocations()) {
+    formatters <- c(
+      formatters,
+      # Artifactory (old versions / configurations)
+      function(repo) {
+        file.path(repo, "src/contrib/Archive", pkgRecord$name, pkgRecord$version, pkgSrcFile)
+      },
+
+      # Nexus
+      function(repo) {
+        file.path(repo, "src/contrib", pkgSrcFile)
+      }
+    )
+  }
+
+  foundVersion <- FALSE
+  for (formatter in formatters) {
+    for (repo in repos) {
+      archiveUrl <- formatter(repo)
+      tryCatch({
+        downloadWithRetries(archiveUrl,
+                            destfile = destFile,
+                            mode = "wb", quiet = TRUE)
+        return(TRUE)
+      }, error = function(e) {
+        # ignore error and try next repo/archive path.
+      })
+    }
+  }
+  return(FALSE)
+}
+
 snapshotSources <- function(project, repos, pkgRecords) {
 
   # Don't snapshot packages included in external.packages
@@ -391,16 +432,12 @@ annotatePkgDesc <- function(pkgRecord, project, lib = libDir(project)) {
     content[name] <- records[name]
   }
 
-  # Write it out
-  write_dcf(content, descFile)
-}
-
-# Annotate a set of packages by name.
-annotatePkgs <- function(pkgNames, project, lib = libDir(project)) {
-  records <- searchPackages(lockInfo(project), pkgNames)
-  lapply(records, function(record) {
-    annotatePkgDesc(record, project, lib)
-  })
+  # Write it out using a temporary file so DESCRIPTION is never partial.
+  tmpf <- tempfile(tmpdir = dirname(descFile))
+  write_dcf(content, tmpf)
+  if (!file.rename(tmpf, descFile)) {
+    stop("Unable to rename annotated package DESCRIPTION")
+  }
 }
 
 # Takes a vector of package names, and returns a logical vector that indicates
@@ -433,10 +470,6 @@ installPkg <- function(pkgRecord,
                        repos,
                        lib = libDir(project))
 {
-  pkgSrc <- NULL
-  type <- "built source"
-  needsInstall <- TRUE
-
   # If we're trying to install a package that overwrites a symlink, e.g. for a
   # cached package, we need to move that symlink out of the way (otherwise
   # `install.packages()` or `R CMD INSTALL` will fail with surprising errors,
@@ -451,7 +484,6 @@ installPkg <- function(pkgRecord,
   # NOTE: a symlink that points to a path that doesn't exist
   # will return FALSE when queried by `file.exists()`!
   if (file.exists(pkgInstallPath) || is.symlink(pkgInstallPath)) {
-
     temp <- tempfile(tmpdir = lib)
     file.rename(pkgInstallPath, temp)
     on.exit({
@@ -467,36 +499,20 @@ installPkg <- function(pkgRecord,
   cacheCopyStatus <- new.env(parent = emptyenv())
   copiedFromCache <- restoreWithCopyFromCache(project, pkgRecord, cacheCopyStatus)
   if (copiedFromCache) {
-    type <- cacheCopyStatus$type
-    needsInstall <- FALSE
+    return(cacheCopyStatus$type)
   }
 
   # Try restoring the package from the 'unsafe' cache, if applicable.
   copiedFromUntrustedCache <- restoreWithCopyFromUntrustedCache(project, pkgRecord, cacheCopyStatus)
   if (copiedFromUntrustedCache) {
-    type <- cacheCopyStatus$type
-    needsInstall <- FALSE
+    return(cacheCopyStatus$type)
   }
 
-  # if we still need to attempt an installation at this point,
-  # remove a prior installation / file from library (if necessary).
-  # we move the old directory out of the way temporarily, and then
-  # delete if if all went well, or restore it if installation failed
-  # for some reason
-  if (needsInstall && file.exists(pkgInstallPath)) {
-    pkgRenamePath <- tempfile(tmpdir = lib)
-    file.rename(pkgInstallPath, pkgRenamePath)
-    on.exit({
-      if (file.exists(pkgInstallPath))
-        unlink(pkgRenamePath, recursive = !is.symlink(pkgRenamePath))
-      else
-        file.rename(pkgRenamePath, pkgInstallPath)
-    }, add = TRUE)
-  }
+  type <- "built source"
+  needsInstall <- TRUE
 
   # Try downloading a binary (when appropriate).
-  if (!(copiedFromCache || copiedFromUntrustedCache) &&
-      hasBinaryRepositories() &&
+  if (hasBinaryRepositories() &&
       binaryRepositoriesEnabled() &&
       isFromCranlikeRepo(pkgRecord, repos) &&
       pkgRecord$name %in% availablePackagesBinary(repos = repos)[, "Package"] &&
@@ -531,14 +547,11 @@ installPkg <- function(pkgRecord,
     })
   }
 
-  if (is.null(pkgSrc)) {
+  if (needsInstall) {
     # When installing from github/bitbucket/gitlab or an older version, use the cached source
     # tarball or zip created in snapshotSources
     pkgSrc <- file.path(srcDir(project), pkgRecord$name,
                         pkgSrcFilename(pkgRecord))
-  }
-
-  if (needsInstall) {
 
     if (!file.exists(pkgSrc)) {
       # If the source file is missing, try to download it. (Could happen in the
@@ -598,27 +611,29 @@ installPkg <- function(pkgRecord,
   }
 
   # Annotate DESCRIPTION file so we know we installed it
-  annotatePkgDesc(pkgRecord, project, lib)
+  withCallingHandlers(
+    annotatePkgDesc(pkgRecord, project, lib),
+    error = function(e) {
+      unlink(pkgInstallPath, recursive = TRUE)
+    }
+  )
 
   # copy package into cache if enabled
   if (isUsingCache(project)) {
-    pkgPath <- file.path(lib, pkgRecord$name)
-
     # copy into global cache if this is a trusted package
     if (isTrustedPackage(pkgRecord$name)) {
-      descPath <- file.path(pkgPath, "DESCRIPTION")
+      descPath <- file.path(pkgInstallPath, "DESCRIPTION")
       if (!file.exists(descPath)) {
         warning("cannot cache package: no DESCRIPTION file at path '", descPath, "'")
       } else {
         hash <- hash(descPath)
         moveInstalledPackageToCache(
-          packagePath = pkgPath,
+          packagePath = pkgInstallPath,
           hash = hash,
           cacheDir = cacheLibDir()
         )
       }
     } else {
-      pkgPath <- file.path(lib, pkgRecord$name)
       tarballName <- pkgSrcFilename(pkgRecord)
       tarballPath <- file.path(srcDir(project), pkgRecord$name, tarballName)
       if (!file.exists(tarballPath)) {
@@ -626,7 +641,7 @@ installPkg <- function(pkgRecord,
       } else {
         hash <- hashTarball(tarballPath)
         moveInstalledPackageToCache(
-          packagePath = pkgPath,
+          packagePath = pkgInstallPath,
           hash = hash,
           cacheDir = untrustedCacheLibDir()
         )
